let data;
let AXIS_INSET;
let AXIS_WIDTH;
let AXIS_HEIGHT;
let black;
let white;
let red;
let grey;

function preload() {
  data = loadTable('https://gist.githubusercontent.com/colmpat/c114d07daeca13effe798a930cf1350f/raw/025b703a269c1a9db9726427b6e8aac7372b1d20/A3_neos_histogram.csv', 'csv', 'header');
}

function setup() {
  createCanvas(960, 600);
  angleMode(DEGREES);
  rectMode(CENTER);
  AXIS_INSET = 70;
  AXIS_WIDTH = width - 2 * AXIS_INSET;
  AXIS_HEIGHT = height - 2 * AXIS_INSET;
  black = color(25,50,60);
  white = color(243,247,240);
  red = color(169,63,85);
  grey = color(110,136,148);
}

function draw() {
  background(white);

  title("Number of Near Earth Objects by Diameter", 30);
  drawTicks(AXIS_INSET);
  drawAxis(AXIS_INSET);
  labelAxis("Diameter (in km)", "ln(count)", 20, 5, 4);
  drawBars(8, AXIS_INSET);
  noLoop();
}

function title(title, inset) {
  textSize(22); fill(black);
  text(title, inset, inset);
}

function drawAxis(inset) {
  stroke(black); strokeWeight(3);
  //top left to bottom left
  line(inset, inset, inset, height - inset);
  //bottom left to bottom right
  line(inset, height - inset, width - inset, height - inset);
}

function labelAxis(xLabel, yLabel, inset, numTicksX, numTicksY) {
  textAlign(CENTER, CENTER); textSize(18); noStroke();
  //label x axis
  text(xLabel, width / 2, height - inset);

  //translate and rotate vertically
  translate(inset, height / 2); rotate(-90);
  //label y axis at the origin of translated grid
  text(yLabel, inset, 0);
  resetMatrix();
}

function drawTicks(inset) {

  for(let i = 0; getHeight(exp(i)) <= AXIS_HEIGHT; i += 0.5) {
    let h = getHeight(exp(i));

    if(i % 1 === 0) {
      stroke(black); strokeWeight(2);
      line(inset - 10, height - inset - h, inset + AXIS_WIDTH, height - inset - h);

      textSize(15); noStroke(); textAlign(RIGHT);
      text(i, inset - 10, height - inset - h + 15);
    } else {
      stroke(grey); strokeWeight(2);
      line(inset, height - inset - h, inset + AXIS_WIDTH, height - inset - h);
    }

  }
}

function drawBars(spacing, inset) {
  let runningX = inset + spacing;
  let numRows = data.getRowCount();
  let barWidth = (AXIS_WIDTH - (numRows + 1) * spacing) / numRows;
  runningX += barWidth / 2.0;
  for(let i = 0; i < numRows; i++) {
    let val = data.getColumn(1).at(i);
    let bin = data.getColumn(0).at(i);
    let barHeight = getHeight(val);
    drawBar(runningX, height - inset, barWidth, barHeight, spacing);
    textAlign(CENTER); fill(black); textSize(15);
    text(bin, runningX, height - inset + 15);
    runningX += spacing + barWidth;

  }
}

function drawBar(x, y, width, height, spacing) {
  noStroke(); fill(red); translate(0, (height / -2.0) - 1); //subtracting 3 to account for axis width
  rect(x, y, width, height);
  resetMatrix();
}

function getHeight(originalHeight) {
  let maxTick = ceil(log(max(data.getColumn(1))));
  return (log(originalHeight) * AXIS_HEIGHT) / maxTick; //from ratio: orig / maxTick = new / axis height
}
